[![FactSet](https://raw.githubusercontent.com/factset/enterprise-sdk/main/docs/images/factset-logo.svg)](https://www.factset.com)

# BookBuilder client library for Python

[![API Version](https://img.shields.io/badge/api-v1.13.0-blue)](https://developer.factset.com/api-catalog/bookbuilder-api)
[![PyPi](https://img.shields.io/pypi/v/fds.sdk.BookBuilder/0.1.0)](https://pypi.org/project/fds.sdk.BookBuilder/v/0.1.0)
[![Apache-2 license](https://img.shields.io/badge/license-Apache2-brightgreen.svg)](https://www.apache.org/licenses/LICENSE-2.0)

The BookBuilder API is a powerful tool that combines detailed reports for companies of interest. Save time by using a single source of knowledge instead of relying on multiple APIs to create a presentation ready PDF, also known as a Public Information Book(PIB). </br></br> Consistently create books with the same content/reports for different companies by defining templates with the set of reports saved for re-use. Or, use templates that are created by FactSet for quick use. </br></br> With our powerful API endpoints, you can get the information on companies of interest quickly where and when you need it. </br></br> 

This Python package is automatically generated by the [OpenAPI Generator](https://openapi-generator.tech) project:

- API version: 1.13.0
- SDK version: 0.1.0
- Build package: org.openapitools.codegen.languages.PythonClientCodegen

For more information, please visit [https://developer.factset.com/contact](https://developer.factset.com/contact)

## Requirements

* Python >= 3.7

## Installation

### Poetry

```shell
poetry add fds.sdk.utils fds.sdk.BookBuilder==0.1.0
```

### pip

```shell
pip install fds.sdk.utils fds.sdk.BookBuilder==0.1.0
```

## Usage

1. [Generate authentication credentials](../../../../README.md#authentication).
2. Setup Python environment.
   1. Install and activate python 3.7+. If you're using [pyenv](https://github.com/pyenv/pyenv):

      ```sh
      pyenv install 3.9.7
      pyenv shell 3.9.7
      ```

   2. (optional) [Install poetry](https://python-poetry.org/docs/#installation).
3. [Install dependencies](#installation).
4. Run the following:

> [!IMPORTANT]
> The parameter variables defined below are just examples and may potentially contain non valid values. Please replace them with valid values.

### Example Code

```python
from fds.sdk.utils.authentication import ConfidentialClient

import fds.sdk.BookBuilder
from fds.sdk.BookBuilder.api import books___creation_api
from fds.sdk.BookBuilder.models import *
from dateutil.parser import parse as dateutil_parser
from pprint import pprint

# See configuration.py for a list of all supported configuration parameters.

# Examples for each supported authentication method are below,
# choose one that satisfies your use case.

# (Preferred) OAuth 2.0: FactSetOAuth2
# See https://github.com/FactSet/enterprise-sdk#oauth-20
# for information on how to create the app-config.json file
#
# The confidential client instance should be reused in production environments.
# See https://github.com/FactSet/enterprise-sdk-utils-python#authentication
# for more information on using the ConfidentialClient class
configuration = fds.sdk.BookBuilder.Configuration(
    fds_oauth_client=ConfidentialClient('/path/to/app-config.json')
)

# Basic authentication: FactSetApiKey
# See https://github.com/FactSet/enterprise-sdk#api-key
# for information how to create an API key
# configuration = fds.sdk.BookBuilder.Configuration(
#     username='USERNAME-SERIAL',
#     password='API-KEY'
# )

# Enter a context with an instance of the API client
with fds.sdk.BookBuilder.ApiClient(configuration) as api_client:
    # Create an instance of the API class
    api_instance = books___creation_api.BooksCreationApi(api_client)
    create_book_post_request = CreateBookPostRequest(
        book_name="Company Book for FDS",
        ticker="FDS-US",
        pagination=PaginationOptions(
            cover=True,
            divider=True,
            toc=True,
            footer=True,
            timestamp=False,
        ),
        content=[
            ContentSection(
                section_id="COMPANY_OVERVIEW",
                reports=[
                    "COMPANY_SNAPSHOT_IB",
                ],
            ),
        ],
    ) # CreateBookPostRequest | 

    try:
        # Kicks off request to create a book with reports of your choice
        # example passing only required values which don't have defaults set
        api_response = api_instance.create_book(create_book_post_request)

        pprint(api_response)
    except fds.sdk.BookBuilder.ApiException as e:
        print("Exception when calling BooksCreationApi->create_book: %s\n" % e)

    # # Get response, http status code and response headers
    # try:
    #     # Kicks off request to create a book with reports of your choice
    #     api_response, http_status_code, response_headers = api_instance.create_book_with_http_info(create_book_post_request)


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.BookBuilder.ApiException as e:
    #     print("Exception when calling BooksCreationApi->create_book: %s\n" % e)

    # # Get response asynchronous
    # try:
    #     # Kicks off request to create a book with reports of your choice
    #     async_result = api_instance.create_book_async(create_book_post_request)
    #     api_response = async_result.get()


    #     pprint(api_response)
    # except fds.sdk.BookBuilder.ApiException as e:
    #     print("Exception when calling BooksCreationApi->create_book: %s\n" % e)

    # # Get response, http status code and response headers asynchronous
    # try:
    #     # Kicks off request to create a book with reports of your choice
    #     async_result = api_instance.create_book_with_http_info_async(create_book_post_request)
    #     api_response, http_status_code, response_headers = async_result.get()


    #     pprint(api_response)
    #     pprint(http_status_code)
    #     pprint(response_headers)
    # except fds.sdk.BookBuilder.ApiException as e:
    #     print("Exception when calling BooksCreationApi->create_book: %s\n" % e)

```

### Using Pandas

To convert an API response to a Pandas DataFrame, it is necessary to transform it first to a dictionary.
```python
import pandas as pd

response_dict = api_response.to_dict()['data']

simple_json_response = pd.DataFrame(response_dict)
nested_json_response = pd.json_normalize(response_dict)
```

### Debugging

The SDK uses the standard library [`logging`](https://docs.python.org/3/library/logging.html#module-logging) module.

Setting `debug` to `True` on an instance of the `Configuration` class sets the log-level of related packages to `DEBUG`
and enables additional logging in Pythons [HTTP Client](https://docs.python.org/3/library/http.client.html).

**Note**: This prints out sensitive information (e.g. the full request and response). Use with care.

```python
import logging
import fds.sdk.BookBuilder

logging.basicConfig(level=logging.DEBUG)

configuration = fds.sdk.BookBuilder.Configuration(...)
configuration.debug = True
```

### Configure a Proxy

You can pass proxy settings to the Configuration class:

* `proxy`: The URL of the proxy to use.
* `proxy_headers`: a dictionary to pass additional headers to the proxy (e.g. `Proxy-Authorization`).

```python
import fds.sdk.BookBuilder

configuration = fds.sdk.BookBuilder.Configuration(
    # ...
    proxy="http://secret:password@localhost:5050",
    proxy_headers={
        "Custom-Proxy-Header": "Custom-Proxy-Header-Value"
    }
)
```

### Custom SSL Certificate

TLS/SSL certificate verification can be configured with the following Configuration parameters:

* `ssl_ca_cert`: a path to the certificate to use for verification in `PEM` format.
* `verify_ssl`: setting this to `False` disables the verification of certificates.
  Disabling the verification is not recommended, but it might be useful during
  local development or testing.

```python
import fds.sdk.BookBuilder

configuration = fds.sdk.BookBuilder.Configuration(
    # ...
    ssl_ca_cert='/path/to/ca.pem'
)
```

### Request Retries

In case the request retry behaviour should be customized, it is possible to pass a `urllib3.Retry` object to the `retry` property of the Configuration.

```python
from urllib3 import Retry
import fds.sdk.BookBuilder

configuration = fds.sdk.BookBuilder.Configuration(
    # ...
)

configuration.retries = Retry(total=3, status_forcelist=[500, 502, 503, 504])
```


## Documentation for API Endpoints

All URIs are relative to *https://api.factset.com/book-builder-api/v1*

Class | Method | HTTP request | Description
------------ | ------------- | ------------- | -------------
*BooksCreationApi* | [**create_book**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BooksCreationApi.md#create_book) | **POST** /create-book | Kicks off request to create a book with reports of your choice
*BooksCreationApi* | [**create_book_from_template**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BooksCreationApi.md#create_book_from_template) | **POST** /create-book-from-template | Kicks off request to create a book with template
*BooksFetchApi* | [**download_book**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BooksFetchApi.md#download_book) | **GET** /download-api-book-aws/{book_id} | Retrieves book in PDF format
*BooksFetchApi* | [**get_book_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BooksFetchApi.md#get_book_list) | **GET** /book-list | Check out the books that are in the book library
*CustomDocumentsFetchApi* | [**get_custom_upload_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/CustomDocumentsFetchApi.md#get_custom_upload_list) | **GET** /custom-upload-list | Check out the documents uploaded before to use them in creating books
*CustomDocumentsUploadApi* | [**upload_custom_document**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/CustomDocumentsUploadApi.md#upload_custom_document) | **POST** /upload-custom-document | Upload a custom document
*TemplatesCreationApi* | [**create_template**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/TemplatesCreationApi.md#create_template) | **POST** /create-template | Kick off request to create template with reports of your choice
*TemplatesFetchApi* | [**get_template_list**](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/TemplatesFetchApi.md#get_template_list) | **GET** /template-list | Retrieves the list of templates that are available


## Documentation For Models

 - [BookInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BookInfo.md)
 - [BookProcessingResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BookProcessingResponse.md)
 - [BookProcessingResponseItem](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/BookProcessingResponseItem.md)
 - [ContentSection](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/ContentSection.md)
 - [CreateBookFromTemplatePostRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/CreateBookFromTemplatePostRequest.md)
 - [CreateBookPostRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/CreateBookPostRequest.md)
 - [CreateTemplatePostRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/CreateTemplatePostRequest.md)
 - [EnableBookDownload](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableBookDownload.md)
 - [EnableBookInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableBookInfo.md)
 - [EnableBookInfoFromTemplate](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableBookInfoFromTemplate.md)
 - [EnableBookListInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableBookListInfo.md)
 - [EnableTemplateInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableTemplateInfo.md)
 - [EnableTemplateInfoPost](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/EnableTemplateInfoPost.md)
 - [InvalidCreateBookFromTemplateRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/InvalidCreateBookFromTemplateRequest.md)
 - [InvalidCreateBookRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/InvalidCreateBookRequest.md)
 - [InvalidDownloadBookAwsRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/InvalidDownloadBookAwsRequest.md)
 - [InvalidTemplateRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/InvalidTemplateRequest.md)
 - [InvalidUploadDocumentRequest](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/InvalidUploadDocumentRequest.md)
 - [PaginationOptions](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/PaginationOptions.md)
 - [TemplateContentSection](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/TemplateContentSection.md)
 - [TemplateInfo](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/TemplateInfo.md)
 - [UploadDocumentResponse](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/UploadDocumentResponse.md)
 - [UploadedDocumentsList](https://github.com/FactSet/enterprise-sdk/tree/main/code/python/BookBuilder/v1/docs/UploadedDocumentsList.md)


## Documentation For Authorization


## FactSetApiKey

- **Type**: HTTP basic authentication


## FactSetOAuth2

- **Type**: OAuth
- **Flow**: application
- **Authorization URL**: 
- **Scopes**: N/A


## Notes for Large OpenAPI documents
If the OpenAPI document is large, imports in fds.sdk.BookBuilder.apis and fds.sdk.BookBuilder.models may fail with a
RecursionError indicating the maximum recursion limit has been exceeded. In that case, there are a couple of solutions:

Solution 1:
Use specific imports for apis and models like:
- `from fds.sdk.BookBuilder.api.default_api import DefaultApi`
- `from fds.sdk.BookBuilder.model.pet import Pet`

Solution 2:
Before importing the package, adjust the maximum recursion limit as shown below:
```
import sys
sys.setrecursionlimit(1500)
import fds.sdk.BookBuilder
from fds.sdk.BookBuilder.apis import *
from fds.sdk.BookBuilder.models import *
```

## Contributing

Please refer to the [contributing guide](../../../../CONTRIBUTING.md).

## Copyright

Copyright 2022 FactSet Research Systems Inc

Licensed under the Apache License, Version 2.0 (the "License");
you may not use this file except in compliance with the License.
You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

Unless required by applicable law or agreed to in writing, software
distributed under the License is distributed on an "AS IS" BASIS,
WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
See the License for the specific language governing permissions and
limitations under the License.

